from gaphor.diagram.interactions.interaction import InteractionItem
from gaphor.diagram.interactions.message import MessageItem
from gaphor.diagram.interactions.lifeline import LifelineItem


def _load():
    from gaphor.diagram.interactions import (
        messageconnect,
        interactionsgrouping,
        interactionspropertypages,
    )


_load()
