"""
All Item's defined in the diagram package. This module
makes it easier to load a diagram item.
"""

from gaphor.diagram.actions import *
from gaphor.diagram.classes import *
from gaphor.diagram.components import *
from gaphor.diagram.general import *
from gaphor.diagram.interactions import *
from gaphor.diagram.profiles import *
from gaphor.diagram.states import *
from gaphor.diagram.usecases import *
