
/* Custom styling for tool box */
toolitemgroup > button {
    border-width: 1px 0px 0px 0px;
    border-radius: 0px;
    padding-top: 2px;
    padding-bottom: 2px;
    background-image: none;
    box-shadow: none;
}

#no-item-selected {
    padding: 12px;
}

#elementeditor {
    padding-top: 2px;
}
    
#elementeditor label {
    padding: 2px;
}

#elementeditor > label {
    padding-bottom: 6px;
}
