"""
The diagram package contains items (to be drawn on the diagram), tools
(used for interacting with the diagram) and interfaces (used for adapting the
diagram).
"""

import gi

gi.require_version("PangoCairo", "1.0")

import gaphor.diagram.actions
import gaphor.diagram.classes
import gaphor.diagram.components
import gaphor.diagram.general
import gaphor.diagram.interactions
import gaphor.diagram.profiles
import gaphor.diagram.states
import gaphor.diagram.usecases
