from gaphor.diagram.profiles.extension import ExtensionItem


def _load():
    from gaphor.diagram.profiles import (
        extensionconnect,
        metaclasspropertypage,
        stereotypepage,
    )


_load()
