from gaphor.diagram.usecases.actor import ActorItem
from gaphor.diagram.usecases.extend import ExtendItem
from gaphor.diagram.usecases.include import IncludeItem
from gaphor.diagram.usecases.usecase import UseCaseItem


def _load():
    from gaphor.diagram.usecases import usecaseconnect


_load()
